@@ -330,24 +330,43 @@ class SpanProcessor(BaseProcessor):
             "id_to_classes": id_to_classes,
         }
 
-
     def create_labels(self, batch):
         labels_batch = []
-        for id in range(len(batch['tokens'])):
-            tokens = batch['tokens'][id]
-            classes_to_id = batch['classes_to_id'][id]
-            ner = batch['entities'][id]
+        for i in range(len(batch['tokens'])):
+            tokens = batch['tokens'][i]
+            classes_to_id = batch['classes_to_id'][i]
+            ner = batch['entities'][i]
             num_classes = len(classes_to_id)
-            spans_idx = [(i, i + j) for i in range(len(tokens)) for j in range(self.config.max_width)]
-            dict_lab = self.get_dict(ner, classes_to_id) if ner else defaultdict(int)
-            span_label = torch.LongTensor([dict_lab[i] for i in spans_idx])
+
+            spans_idx = [(start, start + width)
+                         for start in range(len(tokens))
+                         for width in range(self.config.max_width)]
             spans_idx = torch.LongTensor(spans_idx)
-            valid_span_mask = spans_idx[:, 1] > len(tokens) - 1
-            span_label = span_label.masked_fill(valid_span_mask, 0)
-            labels_one_hot = F.one_hot(span_label, num_classes + 1).float()
+
+            span_to_index = {
+                (spans_idx[idx, 0].item(), spans_idx[idx, 1].item()): idx
+                for idx in range(len(spans_idx))
+            }
+
+            labels_one_hot = torch.zeros(len(spans_idx), num_classes + 1, dtype=torch.float)
+
+            lab_flt = []
+            for span in ner:
+                if span[2] in classes_to_id:
+                    lab_flt.append(((span[0], span[1]), classes_to_id[span[2]]))
+
+            for span, class_id in lab_flt:
+                if span in span_to_index:
+                    idx = span_to_index[span]
+                    labels_one_hot[idx, class_id] = 1.0
+
+            valid_span_mask = spans_idx[:, 1] > (len(tokens) - 1)
+            labels_one_hot[valid_span_mask, :] = 0.0
+
             labels_one_hot = labels_one_hot[:, 1:]
+
             labels_batch.append(labels_one_hot)
-        
+
         # Convert the list of tensors to a single tensor
         if len(labels_batch) > 1:
             labels_batch = pad_2d_tensor(labels_batch)
